/*
 *	  $Id: ValuePresentationDialog_ui.cpp,v 1.4 2009-11-06 09:27:04 kinio Exp $
 *	
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <math.h>

#include <QList>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QPrinter>
#include <QMessageBox>
#include <QProcess>
#include <QTabWidget>

#include "../../namespaces/ns_hqed.h"
#include "../Settings_ui.h"
#include "ValuePresentationDialog_ui.h"
// -----------------------------------------------------------------------------

ValuePresentationDialog_ui* ValuePresentationDialog;
// -----------------------------------------------------------------------------

// #brief Constructor TextEditor_ui class.
//
// #param *parent - Pointer to parent.
ValuePresentationDialog_ui::ValuePresentationDialog_ui(QWidget* /*parent*/)
{
     initWindow();
}
// -----------------------------------------------------------------------------

// #brief Destructore TextEditor_ui class.
ValuePresentationDialog_ui::~ValuePresentationDialog_ui(void)
{
     delete sw0p0Layout;
     delete sw0p1Layout;
     //delete sw0p2Layout;
     delete gb1Layout;
     delete gb2Layout;
     delete formLayout;
}
// -----------------------------------------------------------------------------

// #brief Function that sets the initial values of the window class
//
// #return no values returns
void ValuePresentationDialog_ui::initWindow(void)
{
     setupUi(this);
     setWindowFlags(Qt::Window | Qt::WindowMaximizeButtonHint);
     setWindowModality(Qt::ApplicationModal);
     
     setWindowIcon(QIcon(":/all_images/images/mainico.png"));
     hqed::centerWindow(this);
     
     toolButton->setIcon(QIcon(":/all_images/images/reload.png"));
     toolButton_2->setIcon(QIcon(":/all_images/images/Delete.png"));
     
     connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(sourceIndexChange(int)));
     connect(toolButton, SIGNAL(clicked()), this, SLOT(refreshScriptList()));
     connect(toolButton_2, SIGNAL(clicked()), this, SLOT(deleteScript()));
     connect(pushButton, SIGNAL(clicked()), this, SLOT(okClick()));
     connect(pushButton_2, SIGNAL(clicked()), this, SLOT(closeClick()));
     
     layoutForm();
     listScripts();
     sourceIndexChange(0);
}
// -----------------------------------------------------------------------------

// #brief Function sets the form layout
//
// #return No return value.
void ValuePresentationDialog_ui::layoutForm(void)
{
     resize(5,5);
     
     formLayout = new QGridLayout;
     formLayout->addWidget(groupBox,     0, 0, 1, 3);
     formLayout->addWidget(groupBox_2,   1, 0, 1, 3);
     formLayout->addWidget(pushButton,   2, 1);
     formLayout->addWidget(pushButton_2, 2, 2);
     formLayout->setRowStretch(1, 1);
     formLayout->setColumnStretch(0, 1);
     hqed::setupLayout(formLayout);
     setLayout(formLayout);
     
     gb1Layout = new QGridLayout;
     gb1Layout->addWidget(comboBox, 0, 0);
     hqed::setupLayout(gb1Layout);
     groupBox->setLayout(gb1Layout);
     
     gb2Layout = new QGridLayout;
     gb2Layout->addWidget(stackedWidget, 0, 0);
     hqed::setupLayout(gb2Layout);
     groupBox_2->setLayout(gb2Layout);
     
     sw0p0Layout = new QGridLayout;
     sw0p0Layout->addWidget(label, 0, 0);
     hqed::setupLayout(sw0p0Layout);
     stackedWidget->widget(0)->setLayout(sw0p0Layout);
     
     sw0p1Layout = new QGridLayout;
     sw0p1Layout->addWidget(label_2,      0, 0);
     sw0p1Layout->addWidget(comboBox_2,   1, 0, 1, 3);
     sw0p1Layout->addWidget(toolButton,   1, 3);
     sw0p1Layout->addWidget(toolButton_2, 1, 4);
     sw0p1Layout->addWidget(label_3,      2, 0);
     sw0p1Layout->addWidget(pushButton_3, 2, 1);
     sw0p1Layout->setRowStretch(3, 1);
     sw0p1Layout->setColumnStretch(2, 1);
     sw0p1Layout->setSpacing(4);
     hqed::setupLayout(sw0p1Layout);
     stackedWidget->widget(1)->setLayout(sw0p1Layout);
}
// -----------------------------------------------------------------------------

// #brief Function that lists the script files in the ecmascript directory
//
// #return No return value.
void ValuePresentationDialog_ui::listScripts(void)
{
     QString ecmadir = Settings->ecmascriptPath();
     QDir di(ecmadir);
     QFileInfoList il = di.entryInfoList(QStringList() << "*.esp", QDir::Files | QDir::Readable, QDir::Name);
     
     comboBox_2->clear();
     for(int i=0;i<il.size();++i)
          comboBox_2->addItem(il.at(i).fileName());
}
// -----------------------------------------------------------------------------

// #brief Function that refreshes list of the scripts
//
// #return No return value.
void ValuePresentationDialog_ui::refreshScriptList(void)
{
     QString cn = comboBox_2->currentText();
     listScripts();
     int ci = comboBox_2->findText(cn);
     comboBox_2->setCurrentIndex(ci);
}
// -----------------------------------------------------------------------------

// #brief Function that removes selected script file
//
// #return No return value.
void ValuePresentationDialog_ui::deleteScript(void)
{
     int ci = comboBox_2->currentIndex();
     if(ci == -1)
          return;
          
     QString filename = comboBox_2->itemText(ci);
     filename = Settings->ecmascriptPath() + "/" + filename;
     
     // checking if the file exists
     if(!QFile::exists(filename))
     {
          QMessageBox::critical(NULL, "HQEd", "Selected file \'" + filename + "\' does not exists.", QMessageBox::Ok);
          return;
     }
     
     // confirmation of file removing
     if(QMessageBox::warning(NULL, "HQEd", "Do you really want to remove file \'" + filename + "\'?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
          return;
         
     // file removing
     if(QFile::remove(filename))
          QMessageBox::information(NULL, "HQEd", "File \'" + filename + "\' remove successfully.", QMessageBox::Ok);
     else
          QMessageBox::critical(NULL, "HQEd", "Error while removing file \'" + filename + "\'.", QMessageBox::Ok);
     
     // refreshing file list
     listScripts();
}
// -----------------------------------------------------------------------------

// #brief Function that is triggered when user changes the source of presentation service
//
// #param __index - index of the new source
// #return No return value.
void ValuePresentationDialog_ui::sourceIndexChange(int __index)
{
     stackedWidget->setCurrentIndex(__index);
}
// -----------------------------------------------------------------------------

// #brief Function that is triggered when ok button is pressed
//
// #return No return value.
void ValuePresentationDialog_ui::okClick(void)
{
     close();
}
// -----------------------------------------------------------------------------

// #brief Function that is triggered when close button is pressed
//
// #return No return value.
void ValuePresentationDialog_ui::closeClick(void)
{
     close();
}
// -----------------------------------------------------------------------------
