/*
 *	  $Id: hModelList.cpp,v 1.6 2010-01-06 08:07:56 kinio Exp $
 *	
 *     Implementation by Krzysztof Kaczor <kinio4444@gmail.com>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HQEd has been develped within the HeKatE Project,
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HQEd.
 *
 *     HQEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HQEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HQEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "../M/ARD/ARD_LevelList.h"
#include "../M/ARD/ARD_DependencyList.h"
#include "../M/ARD/ARD_AttributeList.h"
#include "../M/ARD/ARD_PropertyList.h"

#include "../M/hTypeList.h"
#include "../M/XTT/XTT_States.h"
#include "../M/XTT/XTT_TableList.h"
#include "../M/XTT/XTT_ConnectionsList.h"
#include "../M/XTT/XTT_AttributeGroups.h"

#include "../namespaces/ns_xtt.h"

#include "../C/MainWin_ui.h"
 
#include "hModelList.h"
// -----------------------------------------------------------------------------

// #brief Default construcotr of the class
hModelList::hModelList(void) : QList<hModel*>()
{
     _activeModelIndex = -1;
     _modelIDindex = 0;
}
// -----------------------------------------------------------------------------

// #brief Destructor of the class
hModelList::~hModelList(void)
{
     while(size())
          delete takeFirst();
}
// -----------------------------------------------------------------------------

// #brief creates uniqe model ID
//
// #return uniqe model ID as string
QString hModelList::createUniqeModelID(void)
{    
     QString result = "hqedmodel" + QString::number(_modelIDindex);
     _modelIDindex++;
     return result;
}
// -----------------------------------------------------------------------------

// #brief creates new model and adds it to the list
//
// #param __setAsActive - indicates if the created model should be active
// #return pointer to the created model
hModel* hModelList::createNewModel(bool __setAsActive)
{
     hModel* newModel = new hModel;
     newModel->setModelID(createUniqeModelID());
     append(newModel);
     connect(newModel, SIGNAL(activte(hModel*)), this, SLOT(onActivate(hModel*)));
     
     if(__setAsActive)
          activateModel(size()-1);
          
     return newModel;
}
// -----------------------------------------------------------------------------

// #brief function that activates given model
//
// #param __index - index of the model
// #return no values return
void hModelList::activateModel(int __index)
{
     if((__index < 0) || (__index >= size()))
          return;
          
     hModel* __model = at(__index);
     
     ardLevelList = __model->ardLevelList();
	ardAttributeList = __model->ardAttributeList();
	ardPropertyList = __model->ardPropertyList();
	ardDependencyList = __model->ardDependencyList();

     States = __model->xttStates();
     OutputStates = __model->xttOutputStates();
     AttributeGroups = __model->xttAttributeGroups();
     TableList = __model->xttTableList();
     ConnectionsList = __model->xttConnectionsList();
     typeList = __model->xttTypeList();

     if(MainWin != NULL)
     {
          MainWin->switchModels(activeModel(), __model);
          xtt::fullVerification();
          xtt::logicalFullVerification();
     }
     
     if(_activeModelIndex > -1)
          at(_activeModelIndex)->deactivate();
     at(__index)->activate();
     
     _activeModelIndex = __index;
}
// -----------------------------------------------------------------------------

// #brief function that deletes model from the list
//
// #param __index - index of the model that we want to delete
// #return no values return
void hModelList::removeModel(int __index)
{
     if(__index == _activeModelIndex)
          return;
     if(__index < _activeModelIndex)
          _activeModelIndex--;

     // closing model
     delete takeAt(__index);
}
// -----------------------------------------------------------------------------

// #brief function that closes model
//
// #param __index - index of the model that we want to close
// #return no values return
void hModelList::closeModel(int __index)
{
     if((__index < 0) || (__index >= size()))
          return;

     at(__index)->flush();              // clears model

     // selecting next active model
     if((size() > 1) && (__index < (size()-1)))
          activateModel(__index+1);

     if((size() > 1) && (__index == (size()-1)))
          activateModel(__index-1);
     if(size() == 1)
          createNewModel(true)->registerModel();
          
     removeModel(__index);              // removing object
}
// -----------------------------------------------------------------------------

// #brief function that closes model
//
// #return no values return
void hModelList::closeActiveModel(void)
{
     if(_activeModelIndex == -1)
          return;
          
     closeModel(_activeModelIndex);
}
// -----------------------------------------------------------------------------

// #brief function that returns pointer to the active model
//
// #return pointer to the active model or null if active model not exists
hModel* hModelList::activeModel(void)
{
     if((_activeModelIndex < 0) || (_activeModelIndex >= size()))
          return NULL;
          
     return at(_activeModelIndex);
}
// -----------------------------------------------------------------------------

// #brief function that returns index of the active model
//
// #return index of the active model, if there is no active model, returns -1
int hModelList::activeModelIndex(void)
{
     return _activeModelIndex;
}
// -----------------------------------------------------------------------------

// #brief function triggered when activate event occurs
//
// #param __pointer - pointer to the object that received signal
// #return no values return
void hModelList::onActivate(hModel* __pointer)
{
     int io = indexOf(__pointer);
     if(io == -1)
          return;

     activateModel(io);
}
// -----------------------------------------------------------------------------

