/**
 * \file	hModelList.h
 * \author	Krzysztof Kaczor kinio4444@gmail.com
 * \date 	02.07.2009
 * \version	1.0
 * \brief	This file contains class definition that represents model container
 * \note This file is a part of HQEd. It has been develped within the HeKatE Project.
 * HQEd is a free software that you can redistribute and/or modify
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License or 
 * (at your option) any later version.
 * You should have received a copy of the GNU General Public License with HQEd.
 * \note HQEd is distributed in hope that will be useful.
 * It has NO ANY WARRANTY, neither implied warranty of
 * MERCHANTABILITY nor FITNESS FOR A PARTICULAR PURPOSE. 
 * \note Copyright (C) 2006-9 by the HeKatE Project.
 * \see GNU General Public License for more details http://www.gnu.org/licenses/
 * \see HeKatE project http://hekate.ia.agh.edu.pl
 */

#ifndef HMODELLISTH
#define HMODELLISTH
// -----------------------------------------------------------------------------

#include <QList>
#include <QObject>

#include "hModel.h"
// -----------------------------------------------------------------------------

/**
* \class 	hModelList
* \author	Krzysztof Kaczor kinio4444@gmail.com
* \date 	02.07.2009
* \brief 	Class definition that represents model container
*/
class hModelList : public QObject, public QList<hModel*>
{
     Q_OBJECT

private:

     int _activeModelIndex;             ///< The index of the active model
     int _modelIDindex;                 ///< Holds the index of the model ID
     
     /// \brief function that deletes model from the list
     ///
     /// \param __index - index of the model that we want to delete
     /// \return no values return
     void removeModel(int __index);

public:

     /// \brief Default construcotr of the class
     hModelList(void);
     
     /// \brief Destructor of the class
     ~hModelList(void);
     
     /// \brief creates new model and adds it to the list
     ///
     /// \param __setAsActive - indicates if the created model should be active
     /// \return pointer to the created model
     hModel* createNewModel(bool __setAsActive);
     
     /// \brief creates uniqe model ID
     ///
     /// \return uniqe model ID as string
     QString createUniqeModelID(void);
     
     /// \brief function that activates given model
     ///
     /// \param __index - index of the model
     /// \return no values return
     void activateModel(int __index);
     
     /// \brief function that returns pointer to the active model
     ///
     /// \return pointer to the active model or null if active model not exists
     hModel* activeModel(void);
     
     /// \brief function that returns index of the active model
     ///
     /// \return index of the active model, if there is no active model, returns -1
     int activeModelIndex(void);
     
     /// \brief function that infroms all the necessary object about model
     ///
     /// \return no values return
     void registerModel(void);
     
     /// \brief function that closes model
     ///
     /// \param __index - index of the model that we want to close
     /// \return no values return
     void closeModel(int __index);
     
     /// \brief function that closes model
     ///
     /// \return no values return
     void closeActiveModel(void);
     
public slots:

     /// \brief function triggered when activate event occurs
     ///
     /// \param __pointer - pointer to the object that received signal
     /// \return no values return
     void onActivate(hModel* __pointer);
};
// -----------------------------------------------------------------------------
#endif
